/*
** rmm.c -- remove a message(s)
**
** This code is Copyright (c) 2002, by the authors of nmh.  See the
** COPYRIGHT file in the root directory of the nmh distribution for
** complete copyright information.
*/

#include <h/mh.h>
#include <h/utils.h>

static struct swit switches[] = {
#define UNLINKSW  0
	{ "unlink", 0 },
#define NUNLINKSW  1
	{ "nounlink", 0 },
#define VERSIONSW  2
	{ "version", 0 },
#define HELPSW  3
	{ "help", 0 },
	{ NULL, 0 }
};


int
main(int argc, char **argv)
{
	int msgnum, unlink_msgs = 0, vecp = 0;
	char *cp, *maildir, *folder = NULL, **vec;
	char buf[BUFSIZ], **argp;
	char **arguments;
	struct msgs_array msgs = { 0, 0, NULL };
	struct msgs *mp;
	pid_t pid;


#ifdef LOCALE
	setlocale(LC_ALL, "");
#endif
	invo_name = mhbasename(argv[0]);

	context_read();

	arguments = getarguments(invo_name, argc, argv, 1);
	argp = arguments;

	/* parse arguments */
	while ((cp = *argp++)) {
		if (*cp == '-') {
			switch (smatch(++cp, switches)) {
			case AMBIGSW:
				ambigsw(cp, switches);
				done(1);
			case UNKWNSW:
				adios(NULL, "-%s unknown\n", cp);

			case HELPSW:
				snprintf(buf, sizeof(buf), "%s [+folder] [msgs] [switches]", invo_name);
				print_help(buf, switches, 1);
				done(1);
			case VERSIONSW:
				print_version(invo_name);
				done(1);

			case UNLINKSW:
				unlink_msgs++;
				continue;
			case NUNLINKSW:
				unlink_msgs = 0;
				continue;
			}
		}
		if (*cp == '+' || *cp == '@') {
			if (folder)
				adios(NULL, "only one folder at a time!");
			else
				folder = getcpy(expandfol(cp));
		} else
			app_msgarg(&msgs, cp);
	}

	if (!msgs.size)
		app_msgarg(&msgs, seq_cur);
	if (!folder)
		folder = getcurfol();
	maildir = toabsdir(folder);

	if (chdir(maildir) == NOTOK)
		adios(maildir, "unable to change directory to");

	/* read folder and create message structure */
	if (!(mp = folder_read(folder)))
		adios(NULL, "unable to read folder %s", folder);

	/* check for empty folder */
	if (mp->nummsg == 0)
		adios(NULL, "no messages in %s", folder);

	/* parse all the message ranges/sequences and set SELECTED */
	for (msgnum = 0; msgnum < msgs.size; msgnum++)
		if (!m_convert(mp, msgs.msgs[msgnum]))
			done(1);
	seq_setprev(mp);  /* set the previous-sequence */


	if (unlink_msgs) {
		/* "remove" the SELECTED messages */
		folder_delmsgs(mp, 1);

		seq_save(mp);
		context_replace(curfolder, folder);
		context_save();
		folder_free(mp);
		done(0);
		return 1;
	}

	/*
	** This is hackish.  If we don't unlink, but refile,
	** then we need to update the current folder in the
	** context so the external program will refile files
	** from the correct directory.
	*/
	context_replace(curfolder, folder);
	context_save();
	fflush(stdout);

	/* remove by refiling. */
	/* Unset the EXISTS flag for each message to be removed */
	for (msgnum = mp->lowsel; msgnum <= mp->hghsel; msgnum++) {
		if (is_selected(mp, msgnum))
			unset_exists(mp, msgnum);
	}

	/* Mark that the sequence information has changed */
	mp->msgflags |= SEQMOD;

	if (mp->numsel+4 > MAXARGS)
		adios(NULL, "more than %d messages for refile exec",
				MAXARGS - 4);
	vec = (char **)mh_xmalloc((size_t)(mp->numsel + 4) * sizeof(*vec));
	vec[vecp++] = "refile";
	vec[vecp++] = "-nolink";
	vec[vecp++] = concat("+", trashfolder, NULL);
	for (msgnum = mp->lowsel; msgnum <= mp->hghsel; msgnum++) {
		if (!is_selected(mp, msgnum)) {
			continue;
		}
		if (!(vec[vecp++] = strdup(m_name(msgnum)))) {
			adios(NULL, "strdup failed");
		}
	}
	vec[vecp] = NULL;

	fflush(stdout);
	switch (pid = fork()) {
	case -1:
		adios("fork", "unable to");

	case 0:
		execvp(*vec, vec);
		fprintf(stderr, "unable to exec ");
		perror(*vec);
		_exit(-1);

	default:
		pidwait(pid, -1);
	}

	folder_free(mp);
	done(0);
	return 1;
}
